#!/bin/bash

## User declared constants

SiteID="bby"
AltID="BBY"
Prof="Radar449/WwWind"
Rass="Radar449/WwTemp"
SfcMet="CsiDatalogger/SurfaceMet"
GpsIwv="GpsTrimble/WaterVapor"

DataDir="./"

### This code is has been updated to handle portions of time where a given 
### hourly observation file may be missing from the archive (caused temporal 
### match error previously). It also handles days for 
### which all hours are missing (caused crash previously).

##-------------------------------------------------------------------------------------##
## User input, interpretation and date loop
##-------------------------------------------------------------------------------------##

read -p "Enter the start date (YYYYMMDD, UTC) for which to retrieve NOAA BBY met data :" startdate
echo "                           "
read -p "Now enter the end date (YYYYMMDD, UTC, not inclusive) for which to retrieve NOAA BBY met data :" enddate


strdt="${startdate}"
for (( strdt="$startdate"; strdt != enddate; )); do

    yrday=$(date "-d ${strdt}" +"%Y%j")
    yrdate=$(date "-d ${strdt}" +"%m%d")
echo ";------------;;------------>"
echo "   "
echo "retrieving date (YYYYJulday) :"$yrday
echo "retrieving date (MMDD) :"$yrdate
echo "   "
echo ";------------;;------------>"


YYYY=$(echo $yrday | cut -c1-4 )
YY=$(echo $yrday | cut -c3-4 )
DDD=$(echo $yrday | cut -c5-7 )

MM=$(echo $yrdate | cut -c1-2 )
DD=$(echo $yrdate | cut -c3-4 )


wget --quiet -r --cut-dirs=8 --no-parent -nH --user=anonymous --password=mc@ucsd.edu ftp://ftp1.esrl.noaa.gov/psd2/data/realtime/${SfcMet}/${SiteID}/${YYYY}/${DDD}/
wget --quiet -r --cut-dirs=8 --no-parent -nH --user=anonymous --password=mc@ucsd.edu ftp://ftp1.esrl.noaa.gov/psd2/data/realtime/${Prof}/${SiteID}/${YYYY}/${DDD}/
wget --quiet -r --cut-dirs=8 --no-parent -nH --user=anonymous --password=mc@ucsd.edu ftp://ftp1.esrl.noaa.gov/psd2/data/realtime/${Rass}/${SiteID}/${YYYY}/${DDD}/

## GPS iwv currently not working. There is an unknown issue in the PSD ftp archive. Until that is resolved, I cannot even be sure of the default format
## for the gps iwv files, and thus cannot proceed to create a null Temp file.
 wget --quiet --user=anonymous --password=mc@ucsd.edu ftp://ftp1.esrl.noaa.gov/psd2/data/realtime/${GpsIwv}/${SiteID}/${YYYY}/${DDD}/${SiteID}${YY}${DDD}.txt

##-------------------------------------------------------------------------------------##
## Data Processing and reformat
##-------------------------------------------------------------------------------------##


# First pre-format the WProf data
shopt -s nullglob
for f in ./${SiteID}*.*w
do
   HH=$(echo $f | cut -c12-13 ) 
   sed -n 12,60p ${f} | awk -v to_add="$HH" '{ print to_add " " $1 "  " $2 "  " $3 "  " $4}' > Temp-${SiteID}-$YYYY$DDD-$HH-retr01.txt
   sed -n 72,120p ${f} | awk -v to_add="$HH" '{ print to_add " " $1 "  " $2 "  " $3 "  " $4}' > ./Temp-${SiteID}-$YYYY$DDD-$HH-retr02.txt
done

# Now pre-format the RASS data
for f in ./${SiteID}*.*t
do
   HH=$(echo $f | cut -c12-13 ) 
   sed -n 12,29p ${f} | awk -v to_add="$HH" '{ print to_add " " $1 "  " $3 "  " $4 "  " $6 "  " $7}' > ./Temp-${SiteID}-$YYYY$DDD-$HH-rass.txt
done

# Pre-format the Sfc-Met data
for f in ./${SiteID}*.*m
do
   HH=$(echo $f | cut -c12-13 ) 
   awk -F, '{ print $3 " " $4 " " $5 "  " $6 "  " $7 "  " $8 "  " $10 "  " $12 "  " $15}' ${f} > ./Temp-${SiteID}-$YYYY$DDD-$HH-sfcmet.txt
done


# Pre-format the GPS-IWV data
## Once I have an example file to work from again, Ill verify that the hour is being added here. I assume its field 2, but we shall see.
 if [ -e  ${SiteID}${YY}${DDD}.gps ]; then
  echo "creating temp gpsiwv file"
  awk -F, '{ print $2 " " $3}' ${SiteID}${YY}${DDD}.gps > ./Temp-${SiteID}-$YYYY$DDD-gpsiwv.txt
  rm ./${SiteID}${YY}${DDD}.gps
 else
  echo "No GPS IWV file found in archive for this date."
  cp /home/mc/Special-NCL-Funcs/Null-BBY-gpsiwv.txt ./Temp-${AltID}-$YYYY$DDD-gpsiwv.txt
 fi






## insert some code here to test for the existence of the Temp hourly files, and if true, do this..
## sfcmet files..
files_sfc=( Temp-${SiteID}-$YYYY$DDD-*-sfcmet.txt )
echo "number sfcmet files = "${#files_sfc[@]}
if (( ${#files_sfc[@]} )); then
 echo "concatenating surface met files"
 cat Temp-${SiteID}-$YYYY$DDD-*-sfcmet.txt > Temp-${AltID}-$YYYY$DDD-sfcmet.txt
 rm Temp-${SiteID}-$YYYY$DDD-*-sfcmet.txt
 rm ./${SiteID}*${DDD}*.*m
else
 echo "No SFC Met files found in archive for this date."
 cp /home/mc/Special-NCL-Funcs/Null-BBY-sfcmet.txt ./Temp-${AltID}-$YYYY$DDD-sfcmet.txt
fi   

## retr01 files..
files_retr01=( Temp-${SiteID}-$YYYY$DDD-*-retr01.txt )
echo "number 449_retr01 files = "${#files_retr01[@]}
if (( ${#files_retr01[@]} )); then
 echo "concatenating 449_retr01 files"
 cat Temp-${SiteID}-$YYYY$DDD-*-retr01.txt > Temp-${AltID}-$YYYY$DDD-retr01.txt
 rm Temp-${SiteID}-$YYYY$DDD-*-retr01.txt
else
 echo "No 449_retr01 files found in archive for this date."
 cp /home/mc/Special-NCL-Funcs/Null-BBY-retr01.txt ./Temp-${AltID}-$YYYY$DDD-retr01.txt
fi 

## retr02 files..
files_retr02=( Temp-${SiteID}-$YYYY$DDD-*-retr02.txt )
echo "number 449_retr02 files = "${#files_retr02[@]}
if (( ${#files_retr02[@]} )); then
 echo "concatenating files_retr02 files"
 cat Temp-${SiteID}-$YYYY$DDD-*-retr02.txt > Temp-${AltID}-$YYYY$DDD-retr02.txt
 rm Temp-${SiteID}-$YYYY$DDD-*-retr02.txt
 rm ./${SiteID}*${DDD}*.*w
else
 echo "No 449_retr02 files found in archive for this date."
 cp /home/mc/Special-NCL-Funcs/Null-BBY-retr02.txt ./Temp-${AltID}-$YYYY$DDD-retr02.txt
 fi

## rass files..
files_rass=( Temp-${SiteID}-$YYYY$DDD-*-rass.txt )
echo "number RASS files = "${#files_rass[@]}
if (( ${#files_rass[@]} )); then
 echo "concatenating files_rass files"
 cat Temp-${SiteID}-$YYYY$DDD-*-rass.txt > Temp-${AltID}-$YYYY$DDD-rass.txt
 rm Temp-${SiteID}-$YYYY$DDD-*-rass.txt
 rm ./${SiteID}*${DDD}*.*t
else
 echo "No RASS files found in archive for this date."
 cp /home/mc/Special-NCL-Funcs/Null-BBY-rass.txt ./Temp-${AltID}-$YYYY$DDD-rass.txt
fi


shopt -u nullglob








cp ./PSD-2-ncdf-with-timegaps.ncl ./kludge.ncl

sed -i -e s/SSS/${AltID}/g kludge.ncl
sed -i -e s/YRR/${YYYY}/g kludge.ncl
sed -i -e s/DYR/${DDD}/g kludge.ncl
sed -i -e s/MNN/${MM}/g kludge.ncl
sed -i -e s/DDY/${DD}/g kludge.ncl

ncl kludge.ncl



##-------------------------------------------------------------------------------------##
## Clean up
##-------------------------------------------------------------------------------------##


  
 
  rm ./Temp-*sfcmet.txt
  rm ./Temp-*retr01.txt
  rm ./Temp-*retr02.txt
  rm ./Temp-*rass.txt
  rm ./Temp-*gpsiwv.txt
  



  strdt="$(date "-d ${strdt} + 1 days" +'%Y%m%d')"
done


