;------------------------------------;;-------------------------->
; Read the PSD met data netcdf files for a range of dates and 
; plot the 449 MHz wind profiler data (wind first, RASS later)
; 
;----------------------------;;---------------------------------->
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/gsn_code.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/gsn_csm.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/contributed.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/contrib/time_axis_labels.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/shea_util.ncl"
;----------------------------;;---------------------------------->

begin

;wktype = "x11"
;wktype@wkWidth = 1000
;wktype@wkHeight = 1000

wktype = "png"
wktype@wkWidth = 3000
wktype@wkHeight = 3000

optStat = True
optStat@PrintStat = True
;-----------------------------;;--------------------------------->
; define and read data
;-----------------------------;;--------------------------------->

;b    = addfile("./CW2BBY-2015-AllPeriphs.nc","r")

fils = systemfunc ("ls ./PSD-Met-BBY*.nc")
a    = addfiles(fils, "r")
ListSetType (a, "cat")

times = a[:]->Times

sond_times = a[:]->SondeTimes

z449 = a[:]->Height_retr1

dims = dimsizes(z449)

ntim = dims(0)
print("numtimes = "+ntim)
nhgt = dims(1)
print("numheights = "+nhgt)





u449 = a[:]->UWind_retr1
v449 = a[:]->VWind_retr1
;statz = stat_dispersion(u449(:,1),optStat)

;print("sonde times are "+cd_calendar(sond_times,-3))
;print(""+u449(:,2))


iwv = a[:]->Gps_Iwv
bflat = a[:]->Bulk_IWV_Flux
precip = a[:]->Precip
spd_sfc = a[:]->Wspd_Sfc
wdir_sfc = a[:]->Wdir_Sfc


iwv = mask(iwv,iwv.lt.0.,False)
optstat = True
optstat@PrintStat = True



print(""+times@units)
;chemtimes = b->Times

;statz = stat_dispersion(times - chemtimes,optstat)

u449!0 = "time"
u449!1 = "lev"
u449&time = times
u449&lev = z449(0,:)

v449!0 = "time"
v449!1 = "lev"
v449&time = times
v449&lev = z449(0,:)

z449!0 = "time"
z449!1 = "lev"
z449&time = times

iwv!0 = "time"
iwv&time = times

bflat!0 = "time"
bflat&time = times

precip!0 = "time"
precip&time = times

spd_sfc!0 = "time"
spd_sfc&time = times

wdir_sfc!0 = "time"
wdir_sfc&time = times

wdir = u449
wdir = (/ wind_direction(u449,v449,1) /)

spd = u449
spd = (/ sqrt(u449^2+v449^2) /)
spd@description = "Horizontal wind from profiling radar retrieval two"

zplot = z449(lev|:, time|:)

upslope = a[0]->Upslope
alngslope = upslope - 90.

print("upslope wind comes from "+upslope+", along slope wind comes from "+alngslope)

up_component = spd*cos((3.141593/180.)*(360. + wdir - upslope))
alg_component = spd*cos((3.141593/180.)*(360. + wdir - alngslope))

copy_VarMeta(spd,alg_component)
copy_VarMeta(spd,up_component)
;printVarSummary(up_component)




;-------------------------------;;------------------------------------>
; Read Sondes
;-------------------------------;;------------------------------------>

usonde = a[:]->Sonde_UWind
vsonde = a[:]->Sonde_VWind
qsonde = a[:]->Sonde_Qv
psonde = a[:]->Sonde_Press
tsonde = a[:]->Sonde_Tc

usonde!0 = "time"
usonde!1 = "lev"
usonde&time = sond_times
nsondes = dimsizes(sond_times)

usonde&lev  = z449(0,:)

vsonde!0 = "time"
vsonde!1 = "lev"
vsonde&time = sond_times
vsonde&lev  = z449(0,:)

qsonde!0 = "time"
qsonde!1 = "lev"
qsonde&time = sond_times
qsonde&lev = z449(0,:)

rhsonde = relhum(tsonde+273.15,qsonde,psonde*100.)
;stats = stat_dispersion(rhsonde,optStat)
;do ii = 0,dimsizes(sond_times)-1
;print("time is "+cd_calendar(sond_times(ii),-3)+" rel hum. is "+rhsonde(ii,46))
;end do

psonde!0 = "time"
psonde!1 = "lev"
psonde&time = sond_times
psonde&lev = z449(0,:)

spd_snd = usonde
spd_snd = (/ sqrt(usonde^2 + vsonde^2) /)

dpsonde = (/center_finite_diff_n(psonde,1.0,False,0,1)/)
dzsonde = (/dpsonde*100./(-9.81*1.)/) ;; hydrostatic approx. (m)



uqsonde = usonde
uqsonde = (/usonde*qsonde*dzsonde/) ;; zonal vapor flux in g / kg / s

vqsonde = vsonde
vqsonde = (/vsonde*qsonde*dzsonde/) ;; meridional vapor flux in g / kg / s


;---------------------------------------->
; Create Sonde Labels

labeldx = (/4,6,8,10/) ;; which sonde #s should be labelled?
niceIVTLbls = (/"~F25~514 ","~F25~736 ","~F25~956 ","Cold ~C~Front"/)


;-------------------------------;;------------------------------------>
; Create 60-min interpolated wv-flux profiles
;-------------------------------;;------------------------------------>

;spd_ts = ;; make baseline 30 minute timeseries with same units as times.

;; then, for each point and each vertical level, find all (449 and sondes) non-missing
;; obs that are valid from +1hr / -1hr of current time. Average these and assign answer.
;; This will have a temporal smoothing effect, and will combine sondes and 449 with equivalent
;; "trust" but weighted according to temporal density. Maybe not best solution, but not too bad. 

;knots = 4
wts = -1.
smth = 0.5
nderiv = 0


;--------------------------------;;------------------------------->
; Create interpolated (from sonde to radar timeseries) wind 
; fields


merged_uu = spd
merged_uu = merged_uu@_FillValue
merged_uu@description = " "

merged_vv = merged_uu

do ii = 0,nhgt-1
 dumu = (/usonde(lev|ii,time|:)/)
 dxes = ind(.not.ismissing(dumu))
 knots = toint(dimsizes(dxes)/2.)
 merged_uu(:,ii) = (/csa1x(usonde&time(dxes),dumu(dxes),wts,knots,smth,nderiv,merged_uu&time)/)
 delete(dxes)
 
 dumv = (/vsonde(lev|ii,time|:)/)
 dxes = ind(.not.ismissing(dumv))
 knots = toint(dimsizes(dxes)/2.)
 merged_vv(:,ii) = (/csa1x(usonde&time(dxes),dumv(dxes),wts,knots,smth,nderiv,merged_uu&time)/)
 delete(dxes)
 
 delete(dumu)
 delete(dumv)
end do 



merged_spd = merged_uu
merged_spd = (/ sqrt(merged_uu^2+merged_vv^2) /)




;--------------------------------;;------------------------------->
; Create interpolated (from sonde to radar timeseries) flux 
; fields

merged_uq = spd
merged_uq = merged_uq@_FillValue
merged_uq@units = "g kg**-1 s**-1"
merged_uq@description = " "

merged_vq = merged_uq


do ii = 0,nhgt-1
 dumu = (/uqsonde(lev|ii,time|:)/)
 dxes = ind(.not.ismissing(dumu))
 knots = toint(dimsizes(dxes)/2.)
 merged_uq(:,ii) = (/csa1x(usonde&time(dxes),dumu(dxes),wts,knots,smth,nderiv,merged_uq&time)/)
 delete(dxes)
 
 dumv = (/vqsonde(lev|ii,time|:)/)
 dxes = ind(.not.ismissing(dumv))
 knots = toint(dimsizes(dxes)/2.)
 merged_vq(:,ii) = (/csa1x(usonde&time(dxes),dumv(dxes),wts,knots,smth,nderiv,merged_uq&time)/)
 delete(dxes)
 
  delete(dumu)
 delete(dumv)
end do 


;-----------------------------------;;------------------------------------------>>
;Now create upslope flux component and along-slope flux component meteograms.


fluxdir = (/ wind_direction(merged_uq,merged_vq,1) /)

fluxmag = (/ sqrt(merged_uq^2+merged_vq^2) /)

merged_upslp_flx = fluxmag*cos((3.141593/180.)*(360. + fluxdir - upslope))
merged_algslp_flx = fluxmag*cos((3.141593/180.)*(360. + fluxdir - alngslope))

copy_VarMeta(merged_uq,merged_upslp_flx)
copy_VarMeta(merged_uq,merged_algslp_flx)

;stats = stat_dispersion(merged_upslp_flx,optStat)

;------------------------------------;;------------------------------->
; Assign plotting resources
;
;------------------------------------;;------------------------------->
wks1 = gsn_open_wks(wktype ,"CW2016-TwinMeteograms-upslp-n-alngslp")
;drawNDCGrid(wks1)

;-----------------------------;;-------------------------------------->
; Find start and end times (20160305 @ 12 UTC and 20160306 @ 09 UTC)
start_time = cd_inv_calendar(2016,3,5,12,0,0,times@units,0)
end_time = cd_inv_calendar(2016,3,6,9,0,0,times@units,0)

print("max x = "+max(u449&time))
print("min x = "+min(u449&time))
print("plot start time = "+start_time)
print("plot end time = "+end_time)


gsn_define_colormap(wks1,"WhBlGrYeRe")     			; choose color map
colors = (/20,30,40,45,50,55,60,65,70,75,80,85,90,95/)
mags   = ispan(6,42,3)*1.






res = True
restick = True


;res@gsnMaximize       = True
res@gsnDraw           = False
res@gsnFrame          = False

res@vpXF                 = 0.5       				; x location
res@vpYF                 = 0.83       				; y location
res@vpWidthF             = 0.37       				; plot width
res@vpHeightF            = 0.45       				; plot height


restick@ttmFormat = "%c %d %h"
restick@ttmAxis = "XB"
restick@ttmMajorStride = 6
time_axis_labels( u449&time, res, restick)


res@tiXAxisFontThicknessF = 2.					; x axis title font thickness
res@tiYAxisFontThicknessF = 2.					; y axis title font thickness
res@tiXAxisString        = "Day/Hour (UTC)"				; x axis title
res@tiYAxisString        = "Height (km)"				; y axis title
res@tiMainFontHeightF        = 0.013



ymin = min(u449&lev)
ymax = max(u449&lev)
xmin = start_time
xmax = end_time

res@tmXUseBottom                  = False
res@tmXTOn						  = True
res@tmXTLabelsOn                  = True
res@tmXTMode                      = "Explicit"
res@tmXTValues                    = sond_times(labeldx)
res@tmXTLabels                    = niceIVTLbls
;res@tmXTLabelAngleF               = -60.
res@tmXTLabelFontHeightF          = 0.012



res@trXMaxF              = xmax
res@trXMinF              = xmin
res@trYMaxF              = ymax
res@trYMinF              = ymin
res@tmXBLabelDeltaF      = -0.25
res22 = res
res33 = res


;------------------------------------;;---------------------------------->
; assign resources for profiler wind-barb plot.
;------------------------------------;;---------------------------------->
title = "~F25~BBY 449 MHz (light) and Sonde (bold) ~C~ Windbarbs"


res@tiMainString         = title
res@tiXAxisFontThicknessF = 2.					; x axis title font thickness
res@tiYAxisFontThicknessF = 2.					; y axis title font thickness
res@tiXAxisString        = "~F25~Day/Hour (UTC)"				; x axis title
res@tiYAxisFont          = 25
res@tiXAxisOn            = True
res@tiYAxisOn            = False
res@tiYAxisString        = "~F25~Height (km)"				; y axis title
res@tiXAxisFontHeightF   = 0.013
res@tiYAxisFontHeightF   = 0.013
res@tmXBLabelFontHeightF = 0.012					; x axis label font height
res@tmYLLabelFontHeightF = 0.012
res@tmYROn               = False 					; y axis tick marks on RHS turned off
res@tmYLOn               = False
;res@tmXTOn               = False
res@tmYLBorderOn         = True
res@tmXBOn               = True
res@tmXBLabelAngleF      = 45.0
;res@trXReverse           = True


res@vcGlyphStyle        = "WindBarb"
res@vcRefMagnitudeF     = 10.
res@vcRefAnnoOn         = False         				; turns off the ref vector
res@vcRefLengthF        = 0.020         				; set length of ref vector
res@vcMinDistanceF      = 0.009
res@vcMapDirection      = False
    ; This is crucially important if you are plotting data without any 
    ; map-referenced metadata.

res@gsnSpreadColors     = True             				; use full colormap
;res@gsnSpreadColorStart = 48                			; don't use added gray
;res@gsnSpreadColorEnd   = -2               				; don't use added gray

;res@vcMonoWindBarbColor = False            				; color barbs by scalar 
;res@vcLevelColors = colors
res@vcMonoWindBarbColor = True   
res@vcWindBarbColor  = "white"
res@vcVectorDrawOrder = "PreDraw"
res@vcLevels      = mags
res@vcLevelSelectionMode = "ExplicitLevels"
res@vcWindBarbLineThicknessF = 0.05       				; wind barb thickness
res@lbOrientation           = "vertical"				; Label Bar orientation
res@lbLabelFontThicknessF = 2.					; Label bar font thickness
res@pmLabelBarWidthF     = .075					; Label bar width

delete( [/u449@description, u449@units, v449@description, v449@units, spd@description, spd@units/])
plot = gsn_csm_vector_scalar(wks1, u449(lev|:,time|:),v449(lev|:,time|:),spd(lev|:,time|:), res)

getvalues plot
   "vcRefLengthF"			  : wblng
   "vcWindBarbLineThicknessF" : wblt
   "vcWindBarbTickLengthF"    : wbtl
   "vcWindBarbTickSpacingF"   : wbts
end getvalues

;print("wind barb shaft len = "+wblng)
;print("wind barb thick = "+wblt)
;print("wind barb tick len = "+wbtl)
;print("wind barb tick space = "+wbts)


;-------------------------------------;;----------------------------------------------->>
; create sondes windbarb overlay using a dummy blank xy plot and the "wmbarb" utility
;-------------------------------------;;----------------------------------------------->>

res22@tmYROn               = False 					; y axis tick marks on RHS turned off
res22@tmXTOn               = False
res22@tmYLOn               = False
res22@tmXBOn               = False
res22@tiXAxisOn            = False
res22@tiYAxisOn            = False


xc = (/xmin,xmax,xmax,xmin,xmin/)
yc = (/ymin,ymin,ymax,ymax,ymin/)
plot22 = gsn_csm_xy(wks1,xc,yc,res22)


z = (/usonde&lev/)
do gg = 0,nsondes-1

x = sond_times(gg)
 if(x.le.start_time.or.x.ge.end_time)then
  continue
 end if
 u = (/usonde(gg,:)/)
 v = (/vsonde(gg,:)/)
 wspd = spd_snd(gg,:)

 print("sonde x = "+cd_calendar(x,-3))


     wmsetp ("wdf", 1)                 ; meteorological dir
     wmsetp("wbs",wblng)
     wmsetp ("blw",wblt*120.)    ; increase line thickness of wind barbs
     wmsetp ("wbd",wbts)
     wmsetp ("wbt",wbtl)
     do hh = 0,dimsizes(z)-1
       if(.not.ismissing(wspd(hh)))then
       
        wmsetp("col", GetFillColorIndex(mags,colors,wspd(hh)))   ; assign barb color based on wind speed value 
        ;if (z(hh).lt.max(u449&lev).and.z(hh).ge.min(u449&lev)) then
           wmbarb(wks1, x, z(hh), u(hh), v(hh))                   ; draw wind barb
        ;end if
       end if
     end do

     delete( [/x,u,v,wspd/])
  end do


;------------------------------------;;---------------------------------->
; create a test isotachs plot.
;------------------------------------;;---------------------------------->
title2 = "Upslope (black) and Along-Slope (red-dashed) ~C~ vapor flux (g kg~S~-1~N~ s~S~-1~N~)"

mags2   = ispan(3,21,3)*1.


res33@vpXF                 = 0.11       				; x location
res33@tiMainString         = title2
res33@tiXAxisFontThicknessF = 2.					; x axis title font thickness
res33@tiYAxisFontThicknessF = 2.					; y axis title font thickness
res33@tiXAxisString        = "~F25~Day/Hour (UTC)"				; x axis title
res33@tiYAxisFont          = 25
res33@tiXAxisOn            = True
res33@tiYAxisString        = "~F25~Height (km)"				; y axis title
res33@tiXAxisFontHeightF   = 0.013
res33@tiYAxisFontHeightF   = 0.013
res33@tmXBLabelFontHeightF = 0.012					; x axis label font height
res33@tmYLLabelFontHeightF = 0.012
res33@tmYROn               = False 					; y axis tick marks on RHS turned off
;res33@tmXTOn               = False
res33@cnLevelSelectionMode = "ExplicitLevels"
res33@cnLevels             = mags2
res33@cnLineColor          = "black"
res33@cnLineDashPattern    = 0
res33@cnLineThicknessF     = 5.0
res33@cnLineLabelInterval  = 2
res33@cnLineLabelFontHeightF = 0.016
res33@cnInfoLabelOn        = False
res33@tmXBLabelAngleF      = 45.0


delete( [/merged_upslp_flx@description, merged_upslp_flx@units, merged_algslp_flx@description, merged_algslp_flx@units/])
plot33 = gsn_csm_contour(wks1, merged_upslp_flx(lev|:,time|:), res33)
;; generate "along-barrier" speed contours
res33@cnLineColor          = "red"
res33@cnLineDashPattern    = 3
res33@cnLineLabelFontColor = "red"
plot34 = gsn_csm_contour(wks1, merged_algslp_flx(lev|:,time|:), res33)

overlay(plot33,plot34)

draw(plot)
draw(plot22)
draw(plot33)


txres = True
txres@txFontHeightF = 0.016
txres@txFontThicknessF = 2.0

gsn_text_ndc(wks1,"~F25~ a",0.46,0.81,txres)
gsn_text_ndc(wks1,"~F25~ b",0.85,0.81,txres)
txres@txFontHeightF = 0.014
gsn_text_ndc(wks1,"~F25~ m s~S~-1~N~",0.92,0.84,txres)
gsn_text_ndc(wks1,"~F25~IVT ~C~(kg m~S~-1~N~ s~S~-1~N~)",0.11,0.86,txres)


frame(wks1)






end
